<?php


namespace FiloBlu\Hotfix\Plugin\Magento\Catalog\Controller\Adminhtml\Product;

/**
 * Class Validate
 * @package FiloBlu\Hotfix\Plugin\Magento\Catalog\Controller\Adminhtml\Product
 */
class Validate
{

    const LOG_NAME = "/var/log/FiloBlu_Hotfix_Image_Cleaner.log";
    const ACTIVE = "hotfix/image_cleaner/active";
    const IMAGE_TAGS = "hotfix/image_cleaner/image_tags";
    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $productMetadata;
    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    private $_resource;
    private $connection;
    private $row_id;
    private $sku;
    private $name;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * Validate constructor.
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     */
    public function __construct(
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata
    )
    {
        $this->_resource = $resource;
        $this->scopeConfig = $scopeConfig;
        $this->productMetadata = $productMetadata;
    }

    /**
     * @param \Magento\Catalog\Controller\Adminhtml\Product\Validate $subject
     * @return \Magento\Catalog\Controller\Adminhtml\Product\Validate
     */
    public function beforeExecute(
        \Magento\Catalog\Controller\Adminhtml\Product\Validate $subject
    )
    {

        if ($this->checkIfToApplyPlugin()) {

            if (!$this->isActive()) {
                return $subject;
            }

            $productData = $subject->getRequest()->getPost('product', []);
            if ($this->thereIsAnyImageToRemove($productData)) {
                $this->executeImageCleaner();
            }
            return $subject;
        } else {

            if ($this->isActive()) {

                $productData = $subject->getRequest()->getPost('product', []);
                if ($this->thereIsAnyImageToRemove($productData)) {
                    $this->executeImageCleaner();

                }
            }
        }
    }

    /**
     * @return bool
     */
    public function checkIfToApplyPlugin()
    {
        if ($this->getVersion() > "2.2.0") {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->productMetadata->getVersion();
    }

    /**
     * @return mixed
     */
    private function isActive()
    {
        return $this->scopeConfig->getValue(self::ACTIVE);
    }

    /**
     * @param $productData
     * @return bool
     */
    private function thereIsAnyImageToRemove($productData)
    {

        $this->sku = $productData['sku'];
        $this->name = $productData['name'];
        if (isset($productData['media_gallery']['images']) && count($productData['media_gallery']['images']) > 0) {
            foreach ($productData['media_gallery']['images'] as $post_image_id => $post_image) {
                if (isset($post_image['removed']) && $post_image['removed'] == 1) {
                    $this->row_id = $post_image['row_id'];
                    return true;
                    break;
                }
            }
        }
        return false;
    }

    /**
     * @return bool
     */
    public function executeImageCleaner()
    {

        if ($this->row_id) {
            $this->writeLog("Detected image removed from the backend! Cleaning images from product gallery '{$this->name}' (SKU: {$this->sku}) (ROW_ID: {$this->row_id}) if storeview != 0");
        } else {
            $this->writeLog("Cleaning images from all products galleries if storeview != 0");
        }

        $image_tags = $this->getImageTagsString();

        if (!$image_tags) {
            $this->writeLog("Can't clean from product gallery '{$this->name}' (SKU: {$this->sku}) (ROW_ID: {$this->row_id}) - missing image tags from config!");
            return false;
        }

        $sql = "DELETE FROM catalog_product_entity_varchar WHERE store_id <> 0 AND attribute_id IN (
                    SELECT attribute_id 
                    FROM eav_attribute 
                    WHERE backend_type = 'varchar' 
                    AND frontend_input = 'media_image' 
                    AND entity_type_id = 4 
                    AND attribute_code IN ({$image_tags})
                ) ";

        if ($this->row_id) {
            $sql .= " AND row_id = ({$this->row_id})";
        }

        //$this->writeLog($sql);
        $this->getConnection()->query($sql);

        $sql = "DELETE FROM catalog_product_entity_media_gallery_value WHERE store_id <> 0 ";

        if ($this->row_id) {
            $sql .= " AND row_id = ({$this->row_id})";
        }

        //$this->writeLog($sql);
        $this->getConnection()->query($sql);

        return true;

    }

    /**
     * @param $msg
     */
    private function writeLog($msg)
    {
        $writer = new \Zend\Log\Writer\Stream(BP . self::LOG_NAME);
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info($msg);
    }

    /**
     * @return string
     */
    private function getImageTagsString()
    {
        $string = $this->scopeConfig->getValue(self::IMAGE_TAGS);
        $array_string_exploded = explode(",", $string);

        if (count($array_string_exploded) == 0) {
            return "";
        }

        foreach ($array_string_exploded as &$item) {
            $item = trim($item);
            $item = "'" . $item . "'";
        }
        $final_string = implode(",", $array_string_exploded);
        return $final_string;
    }

    /**
     * @return \Magento\Framework\DB\Adapter\AdapterInterface
     */
    private function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_write');
        }
        return $this->connection;
    }

    /**
     * @return string
     */
    public function getLogName()
    {
        return self::LOG_NAME;
    }
}
